# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import logging

from allinpay.core.exceptions import AllInPayClientException
from . import api
from .base import BaseClient
from ..core.utils import AllInPayMd5Signer, random_string


logger = logging.getLogger(__name__)


class AllInPayClient(BaseClient):
    """
    通联支付生产环境
    """
    gateway = api.Gateway()
    prescanpay = api.PreScanPay()
    qpay = api.QPay()
    tranx = api.Tranx()
    unitorder = api.UnitOrder()

    def __init__(self, app_id, cus_id, signer_key, timeout=None):
        super(AllInPayClient, self).__init__(timeout)
        self.app_id = app_id
        self.cus_id = cus_id
        self.signer_key = "key=%s" % signer_key

    def add_sign(self, data, random_str_key="randomstr", sign_key="sign"):
        if random_str_key and random_str_key not in data:
            data[random_str_key] = random_string()
        signer = AllInPayMd5Signer(delimiter=b'&', key=self.signer_key)
        for k, v in data.items():
            signer.add_data("%s=%s" % (k, v))
        data[sign_key] = signer.signature
        return data

    def check_sign(self, data, sign_key="sign"):
        sign = None
        signer = AllInPayMd5Signer(delimiter=b'&', key=self.signer_key)
        for k, v in data.items():
            if k == sign_key:
                sign = v
            else:
                signer.add_data("%s=%s" % (k, v))
        if sign != signer.signature:
            raise AllInPayClientException("SIGNAUTHERR", "签名错误")

    def _handle_pre_request(self, method, uri, kwargs):
        # if 'access_token=' in uri or 'access_token' in kwargs.get('params', {}):
        #     raise ValueError("uri参数中不允许有access_token: " + uri)
        # uri = '%s%saccess_token=%s' % (uri, '&' if '?' in uri else '?', self.access_token)
        return method, uri, kwargs

    def _handle_request_except(self, e, func, *args, **kwargs):
        # if e.errcode in (33001, 40001, 42001, 40014):
        #     self.cache.access_token.delete()
        #     if self.auto_retry:
        #         return func(*args, **kwargs)
        raise e


class AllInPayTestClient(AllInPayClient):
    """
    通联支付测试环境
    """
    API_BASE_URL = 'https://test.allinpaygd.com/'
    SYB_API_BASE_URL = 'https://test.allinpaygd.com/'



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from optionaldict import optionaldict

from .base import AllInPayBaseAPI


class Gateway(AllInPayBaseAPI):
    """
    网关支付
    """
    def pay(self, orderid, trxamt, paytype, returl=None, notifyurl=None,
            validtime=720, goodsid=None, goodsinf=None, gateid=None, limitpay=None, charset='UTF-8'):
        """
        网关支付 - 订单提交接口（商户网站->支付网关）
        https://aipboss.allinpay.com/know/devhelp/home.php?id=73

        :param orderid: 商户唯一订单号
        :param trxamt: 付款金额
        :param paytype: 交易类型
        :param returl: 页面跳转同步通知页面路径
        :param notifyurl: 服务器异步通知页面路径
        :param validtime: 有效时间
        :param goodsid: 商品号
        :param goodsinf: 商品描述信息
        :param gateid: 支付银行
        :param limitpay: 支付限制
        :param charset: 参数字符编码集
        """
        if not returl and not notifyurl:
            raise ValueError("returl和notifyurl不能同时为空")
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "orderid": orderid,
            "trxamt": trxamt,
            "paytype": paytype,
            "returl": returl,
            "notifyurl": notifyurl,
            "validtime": validtime,
            "goodsid": goodsid,
            "goodsinf": goodsinf,
            "gateid": gateid,
            "limitpay": limitpay,
            "charset": charset
        })
        self.add_sign(data)
        return self._post('/apiweb/tranx/queryorder', data)

    def query(self, orderid=None, trxid=None):
        """
        网关支付 - 交易查询接口
        https://aipboss.allinpay.com/know/devhelp/home.php?id=73

        :param orderid: 商户订单号
        :param trxid: 平台交易流水
        """
        if not orderid and not trxid:
            raise ValueError("orderid和trxid必填其一")
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "orderid": orderid,
            "trxid": trxid
        })
        self.add_sign(data)
        return self._post('/apiweb/gateway/query', data)

    def refund(self, reqsn, trxamt, orderid=None, trxid=None, notifyurl=None):
        """
        网关支付 - 订单退款接口
        https://aipboss.allinpay.com/know/devhelp/home.php?id=73

        :param reqsn: 商户退款流水
        :param trxamt: 退款金额
        :param orderid: 商户订单号
        :param trxid: 平台交易流水
        :param notifyurl: 服务器异步通知页面路径
        """
        if not orderid and not trxid:
            raise ValueError("orderid和trxid必填其一")
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqsn": reqsn,
            "trxamt": trxamt,
            "orderid": orderid,
            "trxid": trxid,
            "notifyurl": notifyurl
        })
        self.add_sign(data)
        return self._post('/apiweb/gateway/refund', data)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from optionaldict import optionaldict

from .base import AllInPayBaseAPI


class PreScanPay(AllInPayBaseAPI):
    """
    网上收银预消费
    """
    def pay(self, reqsn, trxamt, paytype, body=None, validtime=5,
            authcode=None, acct=None, notify_url=None, limit_pay=None,
            sub_appid=None, subbranch=None, cusip=None, version='12', remark=None):
        """
        网上收银预消费 - 扫码预消费
        https://aipboss.allinpay.com/know/devhelp/home.php?id=362

        :param reqsn: 商户交易单号
        :param trxamt: 交易金额
        :param paytype: 交易方式
        :param body: 订单标题
        :param validtime: 有效时间
        :param authcode: 支付授权码
        :param acct: 支付平台用户标识
        :param notify_url: 交易结果通知地址
        :param limit_pay: 支付限制
        :param sub_appid: 微信子appid
        :param subbranch: 门店号
        :param cusip: 终端ip
        :param version: 版本号
        :param remark: 备注
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqsn": reqsn,
            "trxamt": trxamt,
            "paytype": paytype,
            "body": body,
            "validtime": validtime,
            "authcode": authcode,
            "acct": acct,
            "notify_url": notify_url,
            "limit_pay": limit_pay,
            "sub_appid": sub_appid,
            "subbranch": subbranch,
            "cusip": cusip,
            "version": version,
            "remark": remark
        })
        self.add_sign(data)
        return self._post('/apiweb/prescanpay/pay', data)

    def finish(self, reqsn, trxamt, oldtrxid, asinfo=None, version='12'):
        """
        网上收银预消费 - 扫码预消费完成
        https://aipboss.allinpay.com/know/devhelp/home.php?id=363

        :param reqsn: 商户完成交易单号
        :param trxamt: 交易金额
        :param oldtrxid: 预消费交易流水
        :param asinfo: 分账信息
        :param version: 版本号
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqsn": reqsn,
            "trxamt": trxamt,
            "oldtrxid": oldtrxid,
            "asinfo": asinfo,
            "version": version
        })
        self.add_sign(data)
        return self._post('/apiweb/prescanpay/finish', data)

    def cancel(self, reqsn, trxamt, oldtrxid, version='12'):
        """
        网上收银预消费 - 扫码预消费交易回退
        https://aipboss.allinpay.com/know/devhelp/home.php?id=364

        :param reqsn: 商户撤销交易单号
        :param trxamt: 交易金额
        :param oldtrxid: 预消费交易流水
        :param version: 版本号
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqsn": reqsn,
            "trxamt": trxamt,
            "oldtrxid": oldtrxid,
            "version": version
        })
        self.add_sign(data)
        return self._post('/apiweb/prescanpay/cancel', data)

    def refund(self, reqsn, trxamt, oldtrxid,  version='12', remark=None):
        """
        网上收银预消费 - 扫码预消费完成交易退款
        https://aipboss.allinpay.com/know/devhelp/home.php?id=365
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqsn": reqsn,
            "trxamt": trxamt,
            "oldtrxid": oldtrxid,
            "version": version,
            "remark": remark
        })
        self.add_sign(data)
        return self._post('/apiweb/prescanpay/refund', data)

    def query(self, reqsn=None, trxid=None, version="12"):
        """
        网上收银预消费 - 扫码预消费查询
        https://aipboss.allinpay.com/know/devhelp/home.php?id=366

        :param reqsn: 商户预消费订单号
        :param trxid: 平台预消费交易流水
        :param version: 版本号
        """
        if not reqsn and not trxid:
            raise ValueError("reqsn和trxid必填其一")
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqsn": reqsn,
            "trxid": trxid,
            "version": version
        })
        self.add_sign(data)
        return self._post('/apiweb/prescanpay/query', data)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import datetime

from optionaldict import optionaldict

from .base import AllInPayBaseAPI


class QPay(AllInPayBaseAPI):
    """
    快捷支付
    """
    def agreeapply(self, meruserid, accttype, acctno, idno, acctname, mobile,
                   validdate=None, cvv2=None, reqip=None, version='11'):
        """
        快捷支付 - 签约申请
        https://aipboss.allinpay.com/know/devhelp/home.php?id=136

        :param meruserid: 商户用户号
        :param accttype: 卡类型
        :param acctno: 银行卡号
        :param idno: 证件号
        :param acctname: 户名
        :param mobile: 手机号码
        :param validdate: 有效期
        :param cvv2: Cvv2
        :param reqip: 请求ip
        :param version: 版本号
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqip": reqip,
            "version": version,
            "reqtime": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
            "meruserid": meruserid,
            "accttype": accttype,
            "acctno": acctno,
            "idno": idno,
            "acctname": acctname,
            "mobile": mobile,
            "validdate": validdate,
            "cvv2": cvv2
        })
        self.add_sign(data)
        return self._post('/apiweb/qpay/agreeapply', data)

    def agreeconfirm(self, meruserid, accttype, acctno, idno, acctname, mobile, smscode, thpinfo,
                     validdate=None, cvv2=None, reqip=None, version='11'):
        """
        快捷支付 - 签约申请确认
        https://aipboss.allinpay.com/know/devhelp/home.php?id=137

        :param meruserid: 商户用户号
        :param accttype: 卡类型
        :param acctno: 银行卡号
        :param idno: 证件号
        :param acctname: 户名
        :param mobile: 手机号码
        :param smscode: 短信验证码
        :param thpinfo: 交易透传信息
        :param validdate: 有效期
        :param cvv2: Cvv2
        :param reqip: 请求ip
        :param version: 版本号
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqip": reqip,
            "version": version,
            "reqtime": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
            "meruserid": meruserid,
            "accttype": accttype,
            "acctno": acctno,
            "idno": idno,
            "acctname": acctname,
            "mobile": mobile,
            "smscode": smscode,
            "thpinfo": thpinfo,
            "validdate": validdate,
            "cvv2": cvv2
        })
        self.add_sign(data)
        return self._post('/apiweb/qpay/agreeconfirm', data)

    def payapplyagree(self, orderid, agreeid, amount, subject, notifyurl, validtime=None, trxreserve=None, asinfo=None,
                      currency="CNY", reqip=None, version='11'):
        """
        快捷支付 - 商户支付申请
        https://aipboss.allinpay.com/know/devhelp/home.php?id=139

        :param orderid: 商户订单号
        :param agreeid: 协议编号
        :param amount: 订单金额
        :param subject: 订单内容
        :param notifyurl: 交易结果通知地址
        :param validtime: 有效时间
        :param trxreserve: 交易备注
        :param asinfo: 分账信息
        :param currency: 币种
        :param reqip: 请求ip
        :param version: 版本号
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqip": reqip,
            "version": version,
            "reqtime": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
            "orderid": orderid,
            "agreeid": agreeid,
            "amount": amount,
            "subject": subject,
            "notifyurl": notifyurl,
            "validtime": validtime,
            "trxreserve": trxreserve,
            "asinfo": asinfo,
            "currency": currency
        })
        self.add_sign(data)
        return self._post('/apiweb/qpay/payapplyagree', data)

    def payagreeconfirm(self, orderid, agreeid, thpinfo, smscode=None, reqip=None, version='11'):
        """
        快捷支付 - 支付确认
        https://aipboss.allinpay.com/know/devhelp/home.php?id=140

        :param orderid: 订单号
        :param agreeid: 协议编号
        :param thpinfo: 交易透传信息
        :param smscode: 短信验证码
        :param reqip: 请求ip
        :param version: 版本号
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqip": reqip,
            "version": version,
            "reqtime": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
            "orderid": orderid,
            "agreeid": agreeid,
            "thpinfo": thpinfo,
            "smscode": smscode
        })
        self.add_sign(data)
        return self._post('/apiweb/qpay/payagreeconfirm', data)

    def paysmsagree(self, orderid, agreeid=None, thpinfo=None, reqip=None, version='11'):
        """
        快捷支付 - 重新获取支付短信
        https://aipboss.allinpay.com/know/devhelp/home.php?id=141

        :param orderid: 商户订单号
        :param agreeid: 协议编号
        :param thpinfo: 交易透传信息
        :param reqip: 请求ip
        :param version: 版本号
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqip": reqip,
            "version": version,
            "reqtime": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
            "orderid": orderid,
            "agreeid": agreeid,
            "thpinfo": thpinfo
        })
        self.add_sign(data)
        return self._post('/apiweb/qpay/paysmsagree', data)

    def agreequery(self, meruserid, reqip=None, version='11'):
        """
        快捷支付 - 协议查询接口
        https://aipboss.allinpay.com/know/devhelp/home.php?id=213

        :param meruserid: 商户用户号
        :param reqip: 请求ip
        :param version: 版本号
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqip": reqip,
            "version": version,
            "reqtime": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
            "meruserid": meruserid
        })
        self.add_sign(data)
        return self._post('/apiweb/qpay/agreequery', data, result_processor=lambda x: x['agreelist'])

    def unbind(self, agreeid, reqip=None, version='11'):
        """
        快捷支付 - 银行卡解绑
        https://aipboss.allinpay.com/know/devhelp/home.php?id=142

        :param agreeid: 协议编号
        :param reqip: 请求ip
        :param version: 版本号
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqip": reqip,
            "version": version,
            "reqtime": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
            "agreeid": agreeid
        })
        self.add_sign(data)
        return self._post('/apiweb/qpay/unbind', data)

    def cancel(self, orderid, trxamt, oldorderid=None, oldtrxid=None, reqip=None, version='11'):
        """
        快捷支付 - 交易撤销
        https://aipboss.allinpay.com/know/devhelp/home.php?id=143

        :param orderid: 商户退款交易单号
        :param trxamt: 交易金额
        :param oldorderid: 原交易单号
        :param oldtrxid: 原交易流水
        :param reqip: 请求ip
        :param version: 版本号
        """
        if not oldorderid and not oldtrxid:
            raise ValueError("oldorderid和oldtrxid必填其一")
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqip": reqip,
            "version": version,
            "reqtime": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
            "orderid": orderid,
            "trxamt": trxamt,
            "oldorderid": oldorderid,
            "oldtrxid": oldtrxid
        })
        self.add_sign(data)
        return self._post('/apiweb/qpay/cancel', data)

    def refund(self, orderid, trxamt, oldorderid=None, oldtrxid=None, reqip=None, version='11'):
        """
        快捷支付 - 交易退款
        https://aipboss.allinpay.com/know/devhelp/home.php?id=144

        :param orderid: 商户退款交易单号
        :param trxamt: 交易金额
        :param oldorderid: 原交易单号
        :param oldtrxid: 原交易流水
        :param reqip: 请求ip
        :param version: 版本号
        """
        if not oldorderid and not oldtrxid:
            raise ValueError("oldorderid和oldtrxid必填其一")
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqip": reqip,
            "version": version,
            "reqtime": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
            "orderid": orderid,
            "trxamt": trxamt,
            "oldorderid": oldorderid,
            "oldtrxid": oldtrxid
        })
        self.add_sign(data)
        return self._post('/apiweb/qpay/refund', data)

    def query(self, orderid=None, trxid=None, reqip=None, version='11'):
        """
        快捷支付 - 交易查询
        https://aipboss.allinpay.com/know/devhelp/home.php?id=145

        :param orderid: 商户的交易订单号
        :param trxid: 平台交易流水
        :param reqip: 请求ip
        :param version: 版本号
        """
        if not orderid and not trxid:
            raise ValueError("orderid和trxid必填其一")
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqip": reqip,
            "version": version,
            "reqtime": datetime.datetime.now().strftime("%Y%m%d%H%M%S"),
            "orderid": orderid,
            "trxid": trxid
        })
        self.add_sign(data)
        return self._post('/apiweb/qpay/query', data)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from optionaldict import optionaldict

from .base import AllInPayBaseAPI


class Posol(AllInPayBaseAPI):
    """
    协同收银
    """

    def authfin(self, reqsn, trxid, trxamt, orgid=None, version='11'):
        """
        预授权完成

        :param reqsn: 商户预授权完成交易流水号
        :param trxid: 交易单号
        :param trxamt: 交易金额
        :param orgid: 集团商户号
        :param version: 版本号
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "orgid": orgid,
            "reqsn": reqsn,
            "trxid": trxid,
            "trxamt": trxamt,
            "version": version
        })
        self.add_sign(data)
        return self._post("/apiweb/posol/authfin", data)

    def refund(self, reqsn, trxid, trxamt, orgid=None, version='11', remark=None):
        """
        交易退货

        :param reqsn: 商户退货交易流水号
        :param trxid: 交易单号
        :param trxamt: 退款金额
        :param orgid: 集团商户号
        :param version: 版本号
        :param remark: 交易备注
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "orgid": orgid,
            "reqsn": reqsn,
            "trxid": trxid,
            "trxamt": trxamt,
            "version": version,
            "remark": remark
        })
        self.add_sign(data)
        return self._post("/apiweb/posol/refund", data)

    def cancel(self, reqsn, trxid, trxamt, orgid=None, version='11'):
        """
        交易撤销

        :param reqsn: 商户撤销交易流水号
        :param trxid: 交易单号
        :param trxamt: 撤销金额
        :param orgid: 集团商户号
        :param version: 版本号
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "orgid": orgid,
            "reqsn": reqsn,
            "trxid": trxid,
            "trxamt": trxamt,
            "version": version
        })
        self.add_sign(data)
        return self._post("/apiweb/posol/cancel", data)

    def query(self, reqsn=None, trxid=None, orgid=None, version='11', remark=None):
        """
        交易查询

        :param orgid: 集团商户号
        :param reqsn: 订单号
        :param trxid: 收银宝交易流水
        :param version: 版本号
        :param remark: 交易备注
        """
        if not reqsn and not trxid:
            raise ValueError("reqsn和trxid不能同时为空")
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "orgid": orgid,
            "reqsn": reqsn,
            "trxid": trxid,
            "version": version,
            "remark": remark
        })
        self.add_sign(data)
        return self._post("/apiweb/posol/query", data)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import datetime

from optionaldict import optionaldict

from .base import AllInPayBaseAPI


class Trxfile(AllInPayBaseAPI):
    """
    对账文件下载接口
    """

    def get(self, date):
        """
        对账文件下载接口

        :param date: 交易日期
        """
        if isinstance(date, datetime.date):
            date = date.strftime("%Y%m%d")
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "date": date
        })
        self.add_sign(data)
        return self._post("/apiweb/trxfile/get", data, result_processor=lambda x: x['url'])



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from optionaldict import optionaldict

from .base import AllInPayBaseAPI


class Tranx(AllInPayBaseAPI):
    """
    终端订单支付, 当面付订单支付
    """

    def queryorder(self, trxdate=None, orderid=None, trxid=None, termno=None, resendnotify=False):
        """
        终端订单支付-订单交易结果查询
        https://aipboss.allinpay.com/know/devhelp/home.php?id=100
        当面付订单支付-交易查询
        https://aipboss.allinpay.com/know/devhelp/home.php?id=207

        :param trxdate: 交易日期
        :param orderid: 订单号
        :param trxid: 收银宝交易流水
        :param termno: 终端号
        :param resendnotify: 是否重发交易结果通知
        """
        if not orderid and not trxid:
            raise ValueError("orderid和trxid不能同时为空")
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "trxdate": trxdate,
            "orderid": orderid,
            "trxid": trxid,
            "termno": termno,
            "resendnotify": 1 if resendnotify else 0,
        })
        self.add_sign(data)
        return self._post('/apiweb/tranx/queryorder', data)

    def refund(self, reqsn, trxamt, trxdate=None, oldtrxid=None, oldbizseq=None, version="01", remark=None):
        """
        当面付订单支付-交易退款
        https://aipboss.allinpay.com/know/devhelp/home.php?id=208

        :param reqsn: 退款流水号
        :param trxamt: 退款金额
        :param trxdate: 交易日期
        :param oldtrxid: 原交易单号
        :param oldbizseq: 原交易商户单号
        :param version: 接口版本编号
        :param remark: 退款备注
        """
        if not oldtrxid and not oldbizseq:
            raise ValueError("oldtrxid和oldbizseq不能同时为空")

        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "version": version,
            "trxdate": trxdate,
            "reqsn": reqsn,
            "oldtrxid": oldtrxid,
            "oldbizseq": oldbizseq,
            "trxamt": trxamt,
            "remark": remark,
        })
        self.add_sign(data)
        return self._post('/voapiweb/unitorder/refund', data)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import inspect
import logging
import requests
from six.moves.urllib.parse import urljoin, urlencode

from .api.base import AllInPayBaseAPI
from ..core.exceptions import AllInPayClientException
from ..core.utils import json_loads


logger = logging.getLogger(__name__)


def _is_api_endpoint(obj):
    return isinstance(obj, AllInPayBaseAPI)


class BaseClient(object):

    _http = requests.Session()

    API_BASE_URL = 'https://vsp.allinpay.com/'
    SYB_API_BASE_URL = 'https://syb.allinpay.com/'

    def __new__(cls, *args, **kwargs):
        self = super(BaseClient, cls).__new__(cls)
        api_endpoints = inspect.getmembers(self, _is_api_endpoint)
        for name, api in api_endpoints:
            api_cls = type(api)
            api = api_cls(self)
            setattr(self, name, api)
        return self

    def __init__(self, timeout=None):
        self.timeout = timeout

    def _request(self, method, url_or_endpoint, **kwargs):
        if not url_or_endpoint.startswith(('http://', 'https://')):
            api_base_url = kwargs.pop('api_base_url', self.API_BASE_URL)
            url = urljoin(api_base_url, url_or_endpoint)
        else:
            url = url_or_endpoint

        if 'params' not in kwargs:
            kwargs['params'] = {}
        if isinstance(kwargs.get('data', ''), dict):
            kwargs['data'] = urlencode(kwargs['data'])
            if 'headers' not in kwargs:
                kwargs['headers'] = {}
            kwargs['headers']['Content-Type'] = 'application/x-www-form-urlencoded'

        kwargs['timeout'] = kwargs.get('timeout', self.timeout)
        result_processor = kwargs.pop('result_processor', None)
        res = self._http.request(
            method=method,
            url=url,
            **kwargs
        )
        try:
            res.raise_for_status()
        except requests.RequestException as reqe:
            logger.error("\n【请求地址】: %s\n【请求参数】：%s \n%s\n【异常信息】：%s",
                         url, kwargs.get('params', ''), kwargs.get('data', ''), reqe)
            raise AllInPayClientException(
                errcode=None,
                errmsg=None,
                client=self,
                request=reqe.request,
                response=reqe.response
            )

        result = self._handle_result(res, method, url, result_processor, **kwargs)

        logger.debug("\n【请求地址】: %s\n【请求参数】：%s \n%s\n【响应数据】：%s",
                     url, kwargs.get('params', ''), kwargs.get('data', ''), result)
        return result

    def _decode_result(self, res):
        try:
            result = json_loads(res.content.decode('utf-8', 'ignore'), strict=False)
        except (TypeError, ValueError):
            # Return origin response object if we can not decode it as JSON
            logger.debug('Can not decode response as JSON', exc_info=True)
            return res
        return result

    def _handle_result(self, res, method=None, url=None, result_processor=None, **kwargs):
        if not isinstance(res, dict):
            # Dirty hack around asyncio based AsyncWeChatClient
            result = self._decode_result(res)
        else:
            result = res

        if not isinstance(result, dict):
            return result
        if 'sign' in result:
            self.check_sign(result)
        if 'retcode' in result and result['retcode'] != 'SUCCESS':
            retcode = result['retcode']
            retmsg = result.get('retmsg', retcode)

            logger.error("\n【请求地址】: %s\n【请求参数】：%s \n%s\n【错误信息】：%s",
                         url, kwargs.get('params', ''), kwargs.get('data', ''), result)
            raise AllInPayClientException(
                retcode,
                retmsg,
                client=self,
                request=res.request,
                response=res
            )
        if 'trxstatus' in result and result['trxstatus'] != '0000':
            trxstatus = result['trxstatus']
            errmsg = result.get('errmsg', trxstatus)

            logger.error("\n【请求地址】: %s\n【请求参数】：%s \n%s\n【错误信息】：%s",
                         url, kwargs.get('params', ''), kwargs.get('data', ''), result)
            raise AllInPayClientException(
                trxstatus,
                errmsg,
                client=self,
                request=res.request,
                response=res
            )

        return result if not result_processor else result_processor(result)

    def _handle_pre_request(self, method, uri, kwargs):
        return method, uri, kwargs

    def _handle_request_except(self, e, func, *args, **kwargs):
        raise e

    def check_sign(self, data, sign_key="sign"):
        raise NotImplementedError

    def request(self, method, uri, **kwargs):
        method, uri_with_access_token, kwargs = self._handle_pre_request(method, uri, kwargs)
        try:
            return self._request(method, uri_with_access_token, **kwargs)
        except AllInPayClientException as e:
            return self._handle_request_except(e, self.request, method, uri, **kwargs)

    def get(self, uri, params=None, **kwargs):
        """
        get 接口请求

        :param uri: 请求url
        :param params: get 参数（dict 格式）
        """
        if params is not None:
            kwargs['params'] = params
        return self.request('GET', uri, **kwargs)

    def post(self, uri, data=None, params=None, **kwargs):
        """
        post 接口请求

        :param uri: 请求url
        :param data: post 数据
        :param params: post接口中url问号后参数（dict 格式）
        """
        if data is not None:
            kwargs['data'] = data
        if params is not None:
            kwargs['params'] = params
        return self.request('POST', uri, **kwargs)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import json

import six
from optionaldict import optionaldict

from .base import AllInPayBaseAPI


class UnitOrder(AllInPayBaseAPI):
    """
    网上收银统一下单
    """

    def pay(
            self, reqsn, trxamt, paytype, acct=None, body=None, notify_url=None, limit_pay=None,
            sub_appid=None, goods_tag=None, benefitdetail=None, chnlstoreid=None, subbranch=None,
            extendparams=None, cusip=None, idno=None, truename=None, asinfo=None, fqnum=None,
            validtime=5, version="11", remark=None
    ):
        """
        网上收银统一下单-统一支付接口
        https://aipboss.allinpay.com/know/devhelp/home.php?id=88

        :param reqsn: 商户交易单号
        :param trxamt: 交易金额
        :param paytype: 交易方式
        :param acct: 支付平台用户标识
        :param body: 订单标题
        :param notify_url: 交易结果通知地址
        :param limit_pay: 支付限制
        :param sub_appid: 微信子appid
        :param goods_tag: 订单优惠标识
        :param benefitdetail: 优惠信息
        :param chnlstoreid: 渠道门店编号
        :param subbranch: 门店号
        :param extendparams: 拓展参数
        :param cusip: 终端ip
        :param idno: 证件号
        :param truename: 付款人真实姓名
        :param asinfo: 分账信息
        :param fqnum: 花呗分期
        :param validtime: 有效时间
        :param version: 版本号
        :param remark: 备注
        """
        if extendparams is not None and isinstance(extendparams, six.string_types):
            extendparams = json.dumps(extendparams)
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqsn": reqsn,
            "trxamt": trxamt,
            "paytype": paytype,
            "acct": acct,
            "body": body,
            "notify_url": notify_url,
            "limit_pay": limit_pay,
            "sub_appid": sub_appid,
            "goods_tag": goods_tag,
            "benefitdetail": benefitdetail,
            "chnlstoreid": chnlstoreid,
            "subbranch": subbranch,
            "extendparams": extendparams,
            "cusip": cusip,
            "idno": idno,
            "truename": truename,
            "asinfo": asinfo,
            "fqnum": fqnum,
            "validtime": validtime,
            "version": version,
            "remark": remark
        })
        self.add_sign(data)
        return self._post("/apiweb/unitorder/pay", data)

    def scanqrpay(
            self, reqsn, trxamt, authcode, body=None, limit_pay=None,
            goods_tag=None, benefitdetail=None, chnlstoreid=None, subbranch=None,
            idno=None, truename=None, asinfo=None, fqnum=None,
            version="11", remark=None
    ):
        """
        网上收银统一统一扫码接口
        https://aipboss.allinpay.com/know/devhelp/home.php?id=88

        :param reqsn: 商户交易单号
        :param trxamt: 交易金额
        :param authcode: 支付授权码
        :param body: 订单标题
        :param limit_pay: 支付限制
        :param goods_tag: 订单优惠标识
        :param benefitdetail: 优惠信息
        :param chnlstoreid: 渠道门店编号
        :param subbranch: 门店号
        :param idno: 证件号
        :param truename: 付款人真实姓名
        :param asinfo: 分账信息
        :param fqnum: 花呗分期
        :param version: 版本号
        :param remark: 备注
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqsn": reqsn,
            "trxamt": trxamt,
            "authcode": authcode,
            "body": body,
            "limit_pay": limit_pay,
            "goods_tag": goods_tag,
            "benefitdetail": benefitdetail,
            "chnlstoreid": chnlstoreid,
            "subbranch": subbranch,
            "idno": idno,
            "truename": truename,
            "asinfo": asinfo,
            "fqnum": fqnum,
            "version": version,
            "remark": remark
        })
        self.add_sign(data)
        return self._post("/apiweb/unitorder/scanqrpay", data)

    def cancel(self, reqsn, trxamt, oldtrxid=None, oldreqsn=None, version="12"):
        """
        网上收银统一下单 - 交易撤销
        https://aipboss.allinpay.com/know/devhelp/home.php?id=91
        H5收银台 - 交易撤销
        https://aipboss.allinpay.com/know/devhelp/home.php?id=315
        手机支付控件 - 交易撤销
        https://aipboss.allinpay.com/know/devhelp/home.php?id=393

        :param reqsn: 商户退款交易单号
        :param trxamt: 交易金额
        :param oldtrxid: 原交易流水
        :param oldreqsn: 原交易单号
        :param version: 版本号
        """
        if not oldreqsn and not oldtrxid:
            raise ValueError("oldtrxid和oldbizseq不能同时为空")

        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqsn": reqsn,
            "trxamt": trxamt,
            "oldtrxid": oldtrxid,
            "oldreqsn": oldreqsn,
            "version": version
        })
        self.add_sign(data)
        return self._post("/apiweb/unitorder/cancel", data)

    def refund(self, reqsn, trxamt, oldtrxid=None, oldreqsn=None, version="12", remark=None):
        """
        网上收银统一下单 - 交易退款
        https://aipboss.allinpay.com/know/devhelp/home.php?id=92
        H5收银台 - 交易退款
        https://aipboss.allinpay.com/know/devhelp/home.php?id=314
        手机支付控件 - 交易退款
        https://aipboss.allinpay.com/know/devhelp/home.php?id=394

        :param reqsn: 商户退款交易单号
        :param trxamt: 交易金额
        :param oldtrxid: 原交易流水
        :param oldreqsn: 原交易单号
        :param version: 版本号
        :param remark: 备注
        """
        if not oldreqsn and not oldtrxid:
            raise ValueError("oldtrxid和oldbizseq不能同时为空")

        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqsn": reqsn,
            "trxamt": trxamt,
            "oldtrxid": oldtrxid,
            "oldreqsn": oldreqsn,
            "remark": remark,
            "version": version
        })
        self.add_sign(data)
        return self._post("/apiweb/unitorder/refund", data)

    def query(self, reqsn, trxid, version="12"):
        """
        网上收银统一下单 - 交易查询
        https://aipboss.allinpay.com/know/devhelp/home.php?id=93
        H5收银台 - 交易查询
        https://aipboss.allinpay.com/know/devhelp/home.php?id=314
        手机支付控件 - 交易查询
        https://aipboss.allinpay.com/know/devhelp/home.php?id=395

        :param reqsn: 商户退款交易单号
        :param trxid: 平台交易流水
        :param version: 版本号
        """
        if not reqsn and not trxid:
            raise ValueError("trxid和reqsn不能同时为空")

        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "reqsn": reqsn,
            "trxid": trxid,
            "version": version
        })
        self.add_sign(data)
        return self._post("/apiweb/unitorder/query", data)

    def authcodetouserid(self, authcode, authtype, sub_appid=None, version="12"):
        """
        网上收银统一下单 - 根据授权码(付款码)获取用户ID
        https://aipboss.allinpay.com/know/devhelp/home.php?id=373

        :param authcode: 授权码（付款码）
        :param authtype: 授权码类型
        :param sub_appid: 微信支付appid
        :param version: 版本号
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "authcode": authcode,
            "authtype": authtype,
            "sub_appid": sub_appid,
            "version": version
        })
        self.add_sign(data)
        return self._post("/apiweb/unitorder/query", data)

    def wxfacepayinfo(self, storeid, storename, subappid, rawdata, deviceid=None, attach=None, version="12"):
        """
        网上收银统一下单 - 根据授权码(付款码)获取用户ID
        https://aipboss.allinpay.com/know/devhelp/home.php?id=406

        :param storeid: 门店编号
        :param storename: 门店名称
        :param subappid: 微信支付appid
        :param rawdata: 初始化数据。由微信人脸SDK的接口返回。
        :param deviceid: 终端设备编号
        :param attach:  附加字段
        :param version: 版本号
        :return:
        """
        if attach is not None and isinstance(attach, six.string_types):
            attach = json.dumps(attach)
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "storeid": storeid,
            "storename": storename,
            "subappid": subappid,
            "rawdata": rawdata,
            "deviceid": deviceid,
            "attach": attach,
            "version": version
        })
        self.add_sign(data)
        return self._post("/apiweb/unitorder/query", data)

    def h5unionpay(self, trxamt, reqsn, returl, notify_url, body,
                   version="12", remark=None, validtime=5, limit_pay=None, asinfo=None):
        """
        H5收银台-订单提交接口
        https://aipboss.allinpay.com/know/devhelp/home.php?id=313

        :param trxamt: 付款金额(单位分)
        :param reqsn: 商户唯一订单号
        :param returl: 页面跳转同步通知页面路径
        :param notify_url: 服务器异步通知页面路径
        :param body: 订单标题
        :param version: 版本号
        :param remark: 订单备注信息
        :param validtime: 有效时间
        :param limit_pay: 支付限制
        :param asinfo: 分账信息
        """
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "trxamt": trxamt,
            "reqsn": reqsn,
            "returl": returl,
            "notify_url": notify_url,
            "body": body,
            "version": version,
            "remark": remark,
            "validtime": validtime,
            "limit_pay": limit_pay,
            "asinfo": asinfo,
        })
        self.add_sign(data)
        return self._post('/apiweb/h5unionpay/unionorder', data, api_base_url=self.SYB_API_BASE_URL)



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

from .unitorder import UnitOrder  # NOQA
from .tranx import Tranx  # NOQA
from .gateway import Gateway  # NOQA
from .prescanpay import PreScanPay  # NOQA
from .qpay import QPay  # NOQA
from .posol import Posol  # NOQA
from .trxfile import Trxfile  # NOQA
from .verify import Verify  # NOQA



# encoding: utf-8
from __future__ import absolute_import, unicode_literals

import datetime

from optionaldict import optionaldict

from .base import AllInPayBaseAPI


class Verify(AllInPayBaseAPI):
    """
    银行账户要素认证
    """
    def _bankverify(self, num, reqsn, cardno, name, idno=None, phone=None, version='11'):
        data = optionaldict({
            "cusid": self.cus_id,
            "appid": self.app_id,
            "version": version,
            "reqtime": datetime.datetime.now().strftime(""),
            "reqsn": reqsn,
            "cardno": cardno,
            "name": name,
            "idno": idno,
            "phone": phone
        })
        self.add_sign(data)
        return self._post("/apiweb/verify/bankverify%d" % num, data, result_processor=lambda x: x['validid'])

    def bankverify2(self, reqsn, cardno, name, version='11'):
        """
        银行账户二要素验证

        :param reqsn: 请求流水
        :param cardno: 银行卡号
        :param name: 户名
        :param version: 版本号
        """
        return self._bankverify(2, reqsn, cardno, name, version=version)

    def bankverify3(self, reqsn, cardno, name, idno, version='11'):
        """
        银行账户三要素验证

        :param reqsn: 请求流水
        :param cardno: 银行卡号
        :param name: 户名
        :param idno: 身份证号
        :param version: 版本号
        """
        return self._bankverify(3, reqsn, cardno, name, idno, version=version)

    def bankverify4(self, reqsn, cardno, name, idno, phone, version='11'):
        """
        银行账户四要素验证

        :param reqsn: 请求流水
        :param cardno: 银行卡号
        :param name: 户名
        :param idno: 身份证号
        :param phone: 手机号码
        :param version: 版本号
        """
        return self._bankverify(4, reqsn, cardno, name, idno, phone, version=version)



#! /usr/bin/env python
# encoding: utf-8


"""A setuptools based setup module.

See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
# To use a consistent encoding
from codecs import open
from os import path
import sys


here = path.abspath(path.dirname(__file__))


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', "Arguments to pass to py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


cmdclass = {}
cmdclass['test'] = PyTest

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

with open('requirements.txt') as f:
    requirements = [l for l in f.read().splitlines() if l]

setup(
    name='pyallinpay',
    version='1.1.0',
    keywords='pyallinpay, SDK, 通联支付',
    description='AllInPay SDK for Python',
    long_description=long_description,
    url='https://github.com/007gzs/pyallinpay',
    author='007gzs',
    author_email='007gzs@sina.com',
    license='LGPL v3',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: '
        'GNU Lesser General Public License v3 (LGPLv3)',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    packages=find_packages(exclude=('tests', )),
    install_requires=requirements,
    zip_safe=False,
    include_package_data=True,
    tests_require=[
        'pytest',
    ],
    cmdclass=cmdclass,
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import six

from .utils import to_binary, to_text


class AllInPayException(Exception):

    def __init__(self, errcode, errmsg):
        """
        :param errcode: Error code
        :param errmsg: Error message
        """
        self.errcode = errcode
        self.errmsg = errmsg

    def __str__(self):
        _repr = 'Error code: {code}, message: {msg}'.format(
            code=self.errcode,
            msg=self.errmsg
        )

        if six.PY2:
            return to_binary(_repr)
        else:
            return to_text(_repr)

    def __repr__(self):
        _repr = '{klass}({code}, {msg})'.format(
            klass=self.__class__.__name__,
            code=self.errcode,
            msg=self.errmsg
        )
        if six.PY2:
            return to_binary(_repr)
        else:
            return to_text(_repr)


class AllInPayClientException(AllInPayException):
    """WeChat API client exception class"""
    def __init__(self, errcode, errmsg, client=None,
                 request=None, response=None):
        super(AllInPayClientException, self).__init__(errcode, errmsg)
        self.client = client
        self.request = request
        self.response = response


class InvalidSignatureException(AllInPayException):
    """Invalid signature exception class"""

    def __init__(self, errcode=-40001, errmsg='Invalid signature'):
        super(InvalidSignatureException, self).__init__(errcode, errmsg)


class InvalidCorpIdOrSuiteKeyException(AllInPayException):
    """Invalid app_id exception class"""

    def __init__(self, errcode=-40005, errmsg='Invalid CorpIdOrSuiteKey'):
        super(InvalidCorpIdOrSuiteKeyException, self).__init__(errcode, errmsg)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

import copy
import hashlib
import json
import random
import string

import six


class ObjectDict(dict):
    """Makes a dictionary behave like an object, with attribute-style access.
    """

    def __getattr__(self, key):
        if key in self:
            return self[key]
        return None

    def __setattr__(self, key, value):
        self[key] = value


class AllInPaySigner(object):
    """AllInPay data signer"""

    def __init__(self, delimiter=b'', key=None):
        self._key = key
        self._data = []
        self._delimiter = to_binary(delimiter)

    def add_data(self, *args):
        """Add data to signer"""
        for data in args:
            self._data.append(to_binary(data))


class AllInPayMd5Signer(AllInPaySigner):

    @property
    def signature(self):
        """Get data signature"""
        data = copy.copy(self._data)
        if self._key:
            data.append(to_binary(self._key))
        data.sort()
        str_to_sign = self._delimiter.join(data)
        return hashlib.md5(str_to_sign).hexdigest().upper()


def to_text(value, encoding='utf-8'):
    """Convert value to unicode, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return ''
    if isinstance(value, six.text_type):
        return value
    if isinstance(value, six.binary_type):
        return value.decode(encoding)
    return six.text_type(value)


def to_binary(value, encoding='utf-8'):
    """Convert value to binary string, default encoding is utf-8

    :param value: Value to be converted
    :param encoding: Desired encoding
    """
    if not value:
        return b''
    if isinstance(value, six.binary_type):
        return value
    if isinstance(value, six.text_type):
        return value.encode(encoding)
    return to_text(value).encode(encoding)


def random_string(length=16):
    rule = string.ascii_letters + string.digits
    rand_list = random.sample(rule, length)
    return ''.join(rand_list)


def byte2int(c):
    if six.PY2:
        return ord(c)
    return c


def json_loads(s, object_hook=ObjectDict, **kwargs):
    return json.loads(s, object_hook=object_hook, **kwargs)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals


class AllInPayBaseAPI(object):

    API_BASE_URL = None
    SYB_API_BASE_URL = None

    def __init__(self, client=None):
        self._client = client
        if self.SYB_API_BASE_URL is None and self._client is not None:
            self.SYB_API_BASE_URL = self._client.SYB_API_BASE_URL

    def _get(self, url, params=None, **kwargs):
        if self.API_BASE_URL and 'api_base_url' not in kwargs:
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.get(url, params, **kwargs)

    def _post(self, url, data=None, params=None, **kwargs):
        if self.API_BASE_URL and 'api_base_url' not in kwargs:
            kwargs['api_base_url'] = self.API_BASE_URL
        return self._client.post(url, data, params, **kwargs)

    def add_sign(self, data, random_str_key="randomstr", sign_key="sign"):
        return self._client.add_sign(data, random_str_key, sign_key)

    @property
    def cus_id(self):
        return self._client.cus_id

    @property
    def app_id(self):
        return self._client.app_id
